#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <io.h>

#include <algorithm>
#include <vector>

class SearchHits
{
private:
	vector<int, allocator<int> >    FnSearchHits;
	string         FstrFileName;
public:
	SearchHits(void)
	{
	}

	SearchHits( string strFileName, int nMaxTerms )
	{
		// Initialize hits to all 0s
		for ( int i=0; i<nMaxTerms; ++i )
			FnSearchHits.insert( FnSearchHits.end(), 0 );

		// Save file name
		FstrFileName = strFileName;


	}
	string GetFileName(void)
	{
		return FstrFileName;
	}
	void SetNumberOfHits( int nTerm, int nHits )
	{
		FnSearchHits[nTerm] = nHits;
	}

	int GetNumberOfTerms(void) const
	{
		int nTerms = 0;
		for ( int i=0; i<FnSearchHits.size(); ++i )
			if ( FnSearchHits[i] != 0 )
				nTerms ++;
		return nTerms;
	}
	int GetNumberOfHits(void) const
	{
		int nTerms = 0;
		for ( int i=0; i<FnSearchHits.size(); ++i )
			nTerms += FnSearchHits[i];
		return nTerms;
	}

	int GetNumberOfHits(int nTerm) const
	{
		return FnSearchHits[ nTerm ];
	}

};

// Sorting for relevance
bool operator < (const SearchHits& sh1, const SearchHits& sh2 )
{
	// If there is a difference, sort on the number of terms.
	// Otherwise, search on the largest number of total hits
	if ( sh2.GetNumberOfTerms() != sh1.GetNumberOfTerms() )
		return sh2.GetNumberOfTerms() < sh1.GetNumberOfTerms();
	else
		return sh2.GetNumberOfHits() < sh1.GetNumberOfHits();

}

// Method to do brute force searching of text in an input string.
int DoFind( string strSearchText, string& text )
{
   int i=0;
   int nCount = 0;

   while ( i < text.size() )
   {
      if ( !strncmp(text.c_str()+i, strSearchText.c_str(), strSearchText.size()) )
      {
         i += strSearchText.size();
         nCount ++;
      }
      else
      {
         i++;
      }
   }
   return nCount;
}

vector< string, allocator<string> > strList;
vector<SearchHits, allocator<SearchHits> > hitList;

int main(int argc, char **argv)
{
   // Check to see that they gave us at least 3 arguments. The first is the
   // program itself (this will always be there). The second is the file mask
   // to do relevance searching on. The third and all others are search terms.
   if ( argc < 3 )
   {
      printf("Usage: search <input-file-mask> <one or more search terms>\n");
      return -1;
   }

   // Build the search list
   for ( int i=2; i<argc; ++i )
   {
	   string strTerm = argv[i];
	   strList.insert( strList.end(), strTerm );
   }


   // Loop through all files that match the file mask given us.
   // Use the Win32 API functions to do this.

   WIN32_FIND_DATA FindFileData;
   HANDLE hFirstFileHandle = FindFirstFile( argv[1], &FindFileData );

   while ( hFirstFileHandle )
   {
	   // Try to open the file for input
	   FILE *fp = fopen(FindFileData.cFileName, "r" );
	   if (fp == NULL)
	   {
		   printf("Unable to open file %s for input\n", argv[1]);
		   if ( FindNextFile( hFirstFileHandle, &FindFileData ) == FALSE )
			   break;
		   continue;
	   }

	   // Search this file one line at a time.
	   char szBuffer[ 256 ];
	   SearchHits hit(FindFileData.cFileName, strList.size());

	   while ( !feof(fp) )
	   {
		  if ( fgets(szBuffer, 255, fp) == NULL )
			 break;
		  string s = szBuffer;

		  // Search each term in the list
		  for ( int i=0; i<strList.size(); ++i )
		  {
			 int nCount = DoFind( strList[i], s );
			 hit.SetNumberOfHits( i, hit.GetNumberOfHits(i)+nCount );
		  }
	   }

	   // Save the results.
	   hitList.insert( hitList.end(), hit );

	   // Close the file and move to the next file in the list.
	   fclose(fp);

	   if ( FindNextFile( hFirstFileHandle, &FindFileData ) == FALSE )
		   break;
   }

   // Sort the list for relevance
   sort( hitList.begin(), hitList.end() );

   // Print out the results for the user.
   for ( i=0; i<hitList.size(); ++i )
   {
	   SearchHits hit = hitList[i];
	   printf("Total Number of search terms found in file %s: %d\n", 
		   hit.GetFileName().c_str(),
		   hit.GetNumberOfTerms());

	   // Print out results for each term in the list for this file.
	   for ( int nTerm=0; nTerm<strList.size(); ++nTerm )
	   {
		   printf("Term: <%s> Number of hits: <%d>\n", 
			   strList[nTerm].c_str(), hit.GetNumberOfHits(nTerm));
	   }

   }

   return 0;
}
